import html
from typing import Any

from htmlmin import minify


class _html:
    @staticmethod
    def compress(*args: Any, **kwargs: Any) -> str:
        return minify(*args, **kwargs)

    @staticmethod
    def escape(text: str) -> str:
        """
        >>> _html.escape("<")
        '&lt;'

        Args:
            text:

        Returns:

        """
        return html.escape(text)

    @staticmethod
    def unescape(text: str) -> str:
        """
        >>> _html.unescape("&lt;")
        '<'

        Args:
            text:

        Returns:

        """
        return html.unescape(text)


__all__ = [
    "_html"
]



from typing import Any


class _list:
    @staticmethod
    def split(
            data: list[Any],
            num_parts: int | None = None,
            part_size: int | None = None
    ) -> list[Any]:
        if not data:
            return []

        if num_parts:
            avg_size = len(data) // num_parts
            remainder = len(data) % num_parts

            result = []
            result.extend(
                [data[start:start + avg_size] for start in range(0, len(data) - avg_size - remainder, avg_size)])
            result.append(data[len(data) - avg_size - remainder:])
            return result

        elif part_size:
            return [data[i:i + part_size] for i in range(0, len(data), part_size)]

        else:
            raise ValueError(
                f"Either num_parts: {num_parts!r} or part_size: {part_size!r} must be provided"
            )

    @staticmethod
    def flatten(data: list[Any]) -> list[Any]:
        return sum((_list.flatten(x) if isinstance(x, list) else [x] for x in data), [])


__all__ = [
    "_list"
]



import os
import platform
import zipfile
from typing import Literal, cast


class _file:
    @staticmethod
    def compress_dir_path(
            dir_path: str,
            compress_type: Literal["zip"] = "zip"
    ) -> str | None:
        dir_path = os.path.abspath(dir_path)

        if not os.path.isdir(dir_path):
            return None

        valid_compress_type = ["zip"]
        if compress_type not in valid_compress_type:
            return None

        compress_file_path = os.path.join(
            os.path.dirname(dir_path), os.path.basename(dir_path) + "." + compress_type
        )

        if compress_type == "zip":
            with zipfile.ZipFile(compress_file_path, "w", zipfile.ZIP_DEFLATED) as zf:
                for root, dirs, files in os.walk(dir_path):
                    for file in files:
                        file_path = os.path.join(root, file)
                        rel_file_path = os.path.relpath(file_path, dir_path)
                        zf.write(file_path, arcname=rel_file_path)

            return compress_file_path

        return None

    @staticmethod
    def get_file_paths_and_dir_paths(path: str) -> tuple[list[str], list[str]]:
        file_paths = []
        dir_paths = []

        path = os.path.abspath(path)

        with os.scandir(path) as entries:
            for entry in entries:
                System = Literal["Windows", "Linux", "Darwin"]
                system: System = cast(System, platform.system())
                if system == "Windows":
                    from nt import DirEntry
                elif system == "Linux":
                    from posix import DirEntry
                elif system == "Darwin":
                    from posix import DirEntry
                else:
                    raise TypeError(
                        f"Invalid type for 'system': "
                        f"Expected `Literal[\"Windows\",\"Linux\",\"Darwin\"]`, "
                        f"but got {type(system).__name__!r} (value: {system!r})"
                    )

                entry: DirEntry
                if entry.is_file():
                    file_paths.append(entry.path)
                elif entry.is_dir():
                    dir_paths.append(entry.path)
                    sub_file_paths, sub_dir_paths = _file.get_file_paths_and_dir_paths(entry.path)
                    file_paths.extend(sub_file_paths)
                    dir_paths.extend(sub_dir_paths)

        return file_paths, dir_paths


__all__ = [
    "_file"
]



from http.cookies import SimpleCookie
from typing import Any

import httpx
import requests.structures
import scrapy
from fake_useragent import UserAgent

from utils._cookies import _cookies


# noinspection PyShadowingNames
class _headers:
    @staticmethod
    def get_default(**kwargs: Any) -> dict[str, str]:
        if not kwargs:
            kwargs = dict(platforms=["desktop"])
        user_agent = UserAgent(**kwargs)

        headers = {
            "User-Agent": user_agent.random
        }
        return headers

    @staticmethod
    def get_updated(
            headers: dict[str, str] | None = None,
            default_headers: dict[str, str] | None = None
    ) -> dict[str, str]:
        """
        >>> _headers.get_updated({"a": "1"}, {"a": "2", "b": "1", "c": "3"})
        {'a': '1', 'b': '1', 'c': '3'}

        Args:
            headers:
            default_headers:

        Returns:

        """
        if headers is None:
            headers = {}
        if default_headers is None:
            default_headers = _headers.get_default()

        for k, v in default_headers.items():
            if k not in headers:
                headers[k] = v

        return headers

    @staticmethod
    def response_headers_to_cookies_dict(
            response_headers: scrapy.http.headers.Headers | requests.structures.CaseInsensitiveDict | httpx.Headers
    ) -> dict[str, str] | None:
        if isinstance(response_headers, scrapy.http.headers.Headers):
            cookies = {}
            for header in response_headers.getlist("Set-Cookie"):
                simple_cookie = SimpleCookie()
                simple_cookie.load(header.decode())
                for key, morsel in simple_cookie.items():
                    cookies[key] = morsel.value
        elif isinstance(response_headers, requests.structures.CaseInsensitiveDict):
            cookies = _cookies.str_to_dict(response_headers.get("Set-Cookie"))
        elif isinstance(response_headers, httpx.Headers):
            cookies = _cookies.str_to_dict(response_headers.get("Set-Cookie"))
        else:
            cookies = None
        return cookies


__all__ = [
    "_headers"
]



from abc import ABCMeta
from collections.abc import ItemsView, KeysView, ValuesView
from types import new_class
from typing import NamedTuple, cast

from utils.__string import _string


class _types:
    class ViewClasses(NamedTuple):
        ItemsView: type
        KeysView: type
        ValuesView: type

    @staticmethod
    def create_view_classes(class_name: str) -> ViewClasses:
        def _create_view_classes(base_class: type) -> type:
            name = f"{class_name}{base_class.__name__}"
            return new_class(
                name,
                (base_class,),
                exec_body=lambda ns: ns.update({
                    "__repr__": lambda self: f"{_string.camel_to_snake(name)}({list(self)})"
                })
            )

        return _types.ViewClasses(
            ItemsView=_create_view_classes(ItemsView),
            KeysView=_create_view_classes(KeysView),
            ValuesView=_create_view_classes(ValuesView),
        )

    @staticmethod
    def create_subclasscheck_meta_class(
            meta_class_name: str = "SubclasscheckMeta",
            *,
            required_all_methods: tuple[str, ...] = tuple(),
            required_any_methods: tuple[str, ...] = tuple()
    ) -> type[ABCMeta]:
        return cast(type[ABCMeta], new_class(
            meta_class_name,
            (ABCMeta,),
            exec_body=lambda ns: ns.update({
                "__subclasscheck__": classmethod(
                    lambda cls, subclass:
                    all([callable(getattr(subclass, i, None)) for i in required_all_methods]) and
                    (any([callable(getattr(subclass, i, None)) for i in required_any_methods]) if required_any_methods
                     else True)
                )
            })
        ))


__all__ = [
    "_types"
]



from typing import Any, TypeGuard


class _validators:
    @staticmethod
    def is_bool(obj: Any) -> TypeGuard[bool]:
        return isinstance(obj, bool)

    @staticmethod
    def is_int(obj: Any) -> TypeGuard[int]:
        return isinstance(obj, int) and not isinstance(obj, bool)

    @staticmethod
    def is_str(val: Any) -> TypeGuard[str]:
        return isinstance(val, str)

    @staticmethod
    def is_list(val: Any) -> TypeGuard[list[Any]]:
        return isinstance(val, list)

    @staticmethod
    def is_list_of[T](val: list[Any], ele_type: type[T]) -> TypeGuard[list[T]]:
        return isinstance(val, list) and all(isinstance(i, ele_type) for i in val)


__all__ = [
    "_validators"
]
